import React, { Component } from 'react';
import DeckGL, { PolygonLayer, MapController } from 'deck.gl';
import { StaticMap } from 'react-map-gl';
import Card from '@material-ui/core/Card';
import CardContent from '@material-ui/core/CardContent';
import CardActions from '@material-ui/core/CardActions';
import Button from '@material-ui/core/Button';
import Typography from '@material-ui/core/Typography';

import { get_color_spectrum, get_undef_color } from './utils/lam-station'

const polygon_size = 0.50
const polygon_size_lat = polygon_size * 2
const polygon_size_lon = polygon_size

const color_spectrum = get_color_spectrum().reverse();
const undef_color = get_undef_color();

// Viewport settings
const viewport = {
  longitude: 25.0,
  latitude: 62.217217,
  zoom: 6,
  pitch: 60,
  bearing: 0
};

class PointMap extends Component {

  constructor(props) {
    super(props);
    const mouseover = null;
    const clicked = null;
    this.state = { viewport, mouseover, clicked };
  }

  handleOnHover(e) {
    this.setState(prevState => {
      let mouseover = null;
      if (e.picked) {
        mouseover = e.object.get_id();
        document.body.style.cursor = "pointer";
      } else {
        document.body.style.cursor = "default";
      }
      return { ...prevState, mouseover };
    });
  }

  handleOnClickObject(e) {
    this.setState(prevState => {
      let clicked = null;
      if (e.picked) {
        clicked = e.object.get_id();
      }
      return { ...prevState, clicked };
    });
    return false;
  }

  handleOnClickMap(e) {
    if (e === null) {
      this.setState(prevState => {
        const clicked = null;
        return { ...prevState, clicked };
      });
    }
  }

  getCard(clicked) {

    const station = this.props.points.get(clicked);
    if (!station) {
      return null;
    }

    const name = station.get_name();
    const avg_speed_dir1 = station.get_avg_speed_dir1();
    const avg_speed_dir2 = station.get_avg_speed_dir2();
    const norm_flow_dir1 = station.get_free_flow_speed_dir1();
    const norm_flow_dir2 = station.get_free_flow_speed_dir2();
    const traffic_vol_dir1 = station.get_traffic_vol_dir1();
    const traffic_vol_dir2 = station.get_traffic_vol_dir2();

    const data = [
      {
        h: "Average speed direction 1",
        v: `${avg_speed_dir1} km/h`,
      },
      {
        h: "Average speed direction 2",
        v: `${avg_speed_dir2} km/h`,
      },
      {
        h: "Normal traffic speed direction 1",
        v: `${norm_flow_dir1} km/h`,
      },
      {
        h: "Normal traffic speed direction 2",
        v: `${norm_flow_dir2} km/h`,
      },
      {
        h: "Traffic volume direction 1",
        v: `${traffic_vol_dir1} cars/h`,
      },
      {
        h: "Traffic volume direction 2",
        v: `${traffic_vol_dir2} cars/h`,
      },
    ];

    const styles = {
      card: {
        position: "absolute",
        marginTop: 10,
        marginLeft: 10,
        maxWidth: 345,
        zIndex: 10,
        left: 0,
        top: 0,
      },
      val_div: {
        marginTop: "3px",
      },
    }

    const close_card = (prevState) => {
      clicked = null;
      return { ...prevState, clicked };
    };

    return (
      <Card style={styles.card}>
        <CardContent>
          <Typography gutterBottom variant="headline" component="h3">
            {name}
          </Typography>
          { data.map(d => (
            <div style={styles.val_div}>
              <Typography variant="caption">
                {d.h}
              </Typography>
              <Typography variant="subheading">
                {d.v}
              </Typography>
            </div>
          ))}
        </CardContent>
        <CardActions>
          <Button size="small" color="primary" onClick={() => this.setState(close_card)}>
            Close
          </Button>
        </CardActions>
      </Card>
    );
  }

  getColorInfo(color_spectrum, undef_color) {
    const gradient_str = color_spectrum.join(", ");
    const styles = {
      flex_container: {
        position: "absolute",
        width: "100%",
        display: "flex",
        top: 0,
        justifyContent: "flex-end",
        zIndex: 10,
        pointerEvents: "none",
      },
      container: {
        paddingTop: "10px",
        paddingRight: "50px",
        pointerEvents: "auto",
        zIndex: 10,
      },
      gradient: {
        backgroundImage: `linear-gradient(${gradient_str})`,
        height: "100px",
        width: "10px",
      },
      undef: {
        backgroundColor: undef_color,
        height: "10px",
        width: "10px",
      },
      side_by_side: {
        display: "flex",
        paddingTop: "10px",
      },
      caption_container: {
        display: "flex",
        flexDirection: "column",
        justifyContent: "space-between",
        paddingLeft: "8px",
      },
    };
    return (
      <div style={styles.flex_container}>
        <div style={styles.container}>
          <div style={styles.side_by_side}>
            <div style={styles.gradient} />
            <div style={styles.caption_container}>
              <Typography variant="caption">
                Normal Traffic
              </Typography>
              <span></span>
              <Typography variant="caption">
                Congested Traffic
              </Typography>
            </div>
          </div>
          <div style={styles.side_by_side}>
            <div style={styles.undef} />
            <div style={styles.caption_container}>
              <Typography variant="caption">
                Missing Data
              </Typography>
            </div>
          </div>
        </div>
      </div>
    );
  }

  getTips() {
    const styles = {
      column_flex_container: {
        position: "absolute",
        top: 0,
        height: "100%",
        width: "100%",
        display: "flex",
        justifyContent: "flex-end",
        flexDirection: "column",
        zIndex: 10,
        pointerEvents: "none",
      },
      row_flex_container: {
        display: "flex",
        justifyContent: "flex-end",
      },
      container: {
        paddingRight: "50px",
        paddingBottom: "10px",
        pointerEvents: "auto",
      },
    };
    return (
      <div style={styles.column_flex_container}>
        <div style={styles.row_flex_container}>
          <div style={styles.container}>
            <Typography variant="subheading" color="textSecondary">
              Scroll Wheel = Zoom In Out
            </Typography>
            <Typography variant="subheading" color="textSecondary">
              Mouse Left Click + Drag = Move Map
            </Typography>
            <Typography variant="subheading" color="textSecondary">
              Ctrl + Mouse Left Click + Drag = Rotate/Tilt Map
            </Typography>
          </div>
        </div>
      </div>
    );
  }

  getMoreInfoLink() {
    const styles = {
      flex_container: {
        position: "absolute",
        top: 0,
        height: "100%",
        display: "flex",
        justifyContent: "flex-end",
        flexDirection: "column",
        pointerEvents: "none",
        zIndex: 10,
      },
      container: {
        pointerEvents: "auto",
      },
    };
    return (
      <div style={styles.flex_container}>
        <div style={styles.container}>
          <Button
            href="https://gitlab.labranet.jamk.fi/data-analysis-and-ai/lam-station-visualization/blob/master/README.md"
            size="large"
          >
            More Info
          </Button>
        </div>
      </div>
    );
  }

  render() {
    const point_coords = this.props.points.toArray();
    const layers = [
      new PolygonLayer({
        id: 'lam-points',
        data: point_coords,
        getPosition: d => d.get_coordinates(),
        getFillColor: d => d.get_color(),
        getPolygon: d => {
          let [lat, lon] = d.coordinates;
          let zoom = this.state.viewport.zoom;
          let pol_lat = polygon_size_lat / (2 ** zoom);
          let pol_lon = polygon_size_lon / (2 ** zoom);
          let coords = [
            [lat + -pol_lat, lon + pol_lon],
            [lat + pol_lat, lon + pol_lon],
            [lat + pol_lat, lon + -pol_lon],
            [lat + -pol_lat, lon + -pol_lon]
          ];
          return coords;
        },
        filled: true,
        extruded: true,
        getElevation: d => {
          let zoom = this.state.viewport.zoom;
          // super hacky way of scaling the bar height
          let vol_scale = 20 * (1.5 ** 6);
          return d.get_avg_traffic_vol() * vol_scale / (1.5 ** zoom);
        },
        getLineColor: d => {
          if (this.state.mouseover === d.get_id()) {
            return [255, 0, 0];
          }
          return d.get_color();
        },
        wireframe: true,
        pickable: true,
        onClick: e => this.handleOnClickObject(e),
        onHover: e => this.handleOnHover(e),
      })
    ];

    return (
      <div>
        <DeckGL
          {...this.state.viewport}
          {...this.props.mapSize}
          layers={layers}
          controller={MapController}
          onViewportChange={v => this.setState({viewport: v})}
          onLayerClick={e => this.handleOnClickMap(e)}
        >
          <StaticMap
            {...this.props.mapSize}
            mapStyle="mml-basemap.json"
            viewState={this.state.viewport}
          />
        </DeckGL>
        { this.state.clicked &&
            this.getCard(this.state.clicked)
        }
        {this.getColorInfo(color_spectrum, undef_color)}
        {this.getTips()}
        {this.getMoreInfoLink()}
      </div>
    );
  }

}

export default PointMap;
