import { Map } from 'immutable';
import mqtt from 'mqtt';

import LamStation from './lam-station.js';

export function get_lam_points(api_url, set_state) {
  return fetch(`${api_url}/api/v1/metadata/tms-stations`)
    .then((res) => res.json())
    .then((json) => {
      let stations = Map();
      for (let s of json.features) {
        let id = s.id;
        let coordinates = s.geometry.coordinates;
        let station = new LamStation(id, coordinates, 0, 0, s);
        stations = stations.set(id, station);
      }
      set_state((prevState) => {
        const lam_points = stations;
        return { ...prevState, lam_points };
      });
    })
    .then(() => {
      return Promise.all([
        get_lam_station_data(api_url, set_state),
        get_free_flow_speeds(api_url, set_state),
      ]);
    })
    .catch(e => (console.error(e)));
}

export function get_free_flow_speeds(api_url, set_state) {
  return fetch(`${window.location.href}/free-flow-speeds.json`)
    .then((res) => res.text())
    .then((t) => JSON.parse(t))
    .then((json) => {
      const tms_data = json.tmsFreeFlowSpeeds;
      set_state((prevState) => {
        let lam_points = prevState.lam_points;
        for (let data of tms_data) {
          let id = data.id;
          let point = lam_points.get(id);
          if (!point) {
            continue;
          }
          point.set_free_flow_speed_dir1(data.freeFlowSpeed1);
          point.set_free_flow_speed_dir2(data.freeFlowSpeed2);
          lam_points = lam_points.set(id, point);
        }
        return { ...prevState, lam_points };
      });
    })
    .catch(e => (console.error(e)));
}

export function get_lam_station_data(api_url, set_state) {
  return fetch(`${api_url}/api/v1/data/tms-data`)
    .then((res) => res.json())
    .then((json) => {
      set_state((prevState) => {
        let lam_points = prevState.lam_points;
        for (let station of json.tmsStations) {
          let id = station.id;
          let point = lam_points.get(id);
          if (!point) {
            continue;
          }
          for (let sensor_val of station.sensorValues) {
            let name = sensor_val.name;
            let value = sensor_val.sensorValue;
            if (name === "KESKINOPEUS_5MIN_LIUKUVA_SUUNTA1") {
              point.set_avg_speed_dir1(value);
            } else if (name === "KESKINOPEUS_5MIN_LIUKUVA_SUUNTA2") {
              point.set_avg_speed_dir2(value);
            } else if (name === "OHITUKSET_5MIN_LIUKUVA_SUUNTA1") {
              point.set_traffic_vol_dir1(value);
            } else if (name === "OHITUKSET_5MIN_LIUKUVA_SUUNTA2") {
              point.set_traffic_vol_dir2(value);
            }
          }
          lam_points = lam_points.set(id, point);
        }
        return { ...prevState, lam_points };
      });
    })
    .catch(e => (console.error(e)));
}

export function connect_lam_ws(api_url, update_state) {
  let data_buff = [];
  let url = `${api_url}/mqtt`;
  // The username and password is documented on digitraffic api documentation site here:
  // https://www.digitraffic.fi/tieliikenne/#websocket-rajapinnat
  let client = mqtt.connect(url, {
    username: 'digitraffic',
    password: 'digitrafficPassword',
  });
  client.on('connect', () => {
    console.info(`MQTT connection open to ${url}`);
    const topic = "tms/#";
    client.subscribe(topic, (err) => {
      if (err) {
        console.error(`Subscription error: ${err}`);
      } else {
        console.log(`Subscribed to ${topic}`);
      }
    });
  });
  client.on('error', (err) => {
    console.error(`error: ${err}`);
  });
  client.on('message', (topic, msg) => {
    let data = JSON.parse(msg.toString());
    if (!data) {
      return
    }
    let id = data.roadStationId;
    let value = data.sensorValue;
    let name = data.name;
    if (id && value && name) {
      data_buff.push({ id, value, name });
    }
  });

  function buffer_flusher() {
    return new Promise(resolve => setTimeout(resolve, 500))
      .then(() => {
        update_state((prev_state) => {
          let lam_points = prev_state.lam_points;
          for (let obj of data_buff) {
            let { id, value, name } = obj;
            let point = lam_points.get(id);
            if (!point) {
              console.error(`point not found ${id}`);
              continue;
            }
            if (name === "KESKINOPEUS_5MIN_LIUKUVA_SUUNTA1") {
              point.set_avg_speed_dir1(value);
              lam_points = lam_points.set(id, point);
            } else if (name === "KESKINOPEUS_5MIN_LIUKUVA_SUUNTA2") {
              point.set_avg_speed_dir2(value);
              lam_points = lam_points.set(id, point);
            } else if (name === "OHITUKSET_5MIN_LIUKUVA_SUUNTA1") {
              point.set_traffic_vol_dir1(value);
              lam_points = lam_points.set(id, point);
            } else if (name === "OHITUKSET_5MIN_LIUKUVA_SUUNTA2") {
              point.set_traffic_vol_dir2(value);
              lam_points = lam_points.set(id, point);
            }
          }
          data_buff = [];
          return { lam_points };
        });
      })
      .then(() => {
        buffer_flusher();
      });
  }

  buffer_flusher();
}
